import React, { cloneElement } from 'react';
import PropTypes from 'prop-types';

var HORIZONTAL = "horizontal"; // 水平滚动
var VERTICAL = "vertical"; // 垂直滚动

var noop = function noop() {};
var packRefs = function packRefs() {
  for (var _len = arguments.length, refs = Array(_len), _key = 0; _key < _len; _key++) {
    refs[_key] = arguments[_key];
  }

  return function (node) {
    return refs.forEach(function (ref) {
      if (typeof ref === "function") {
        ref(node);
      } else if (ref) {
        ref.current = node;
      }
    });
  };
};
var setDirectionStyle = function setDirectionStyle(index, direction, speed, containerRect) {
  var width = containerRect.width,
      height = containerRect.height;

  var style = {
    transitionDuration: speed + "ms"
  };
  if (direction === HORIZONTAL) {
    var x = index ? (index - 1) * width : 0;
    style.transform = "translate3d(" + -x + "px, 0px, 0px)";
    style.flexDirection = "row";
  } else if (direction === VERTICAL) {
    var y = (index - 1) * height;
    style.transform = "translate3d(0px, " + -y + "px, 0px)";
    style.flexDirection = "column";
  }
  return style;
};

// 翻转移动的数量单位，让它适合translate
var reversalNumber = function reversalNumber(number) {
  return number > 0 ? -number : Math.abs(number);
};

var getStyle = function getStyle(node, attr) {
  return window.getComputedStyle(node)[attr];
};

// matrix.length => y坐标    matrix.length - 1 => x坐标
var parseMatrix = function parseMatrix(value) {
  return value.match(/-?\d+\.?\d*/g);
};

// 如果移动超过容器一半那么就算作移动一步
var getStep = function getStep(value, width) {
  return Math.floor(Math.abs(value) / (width / 2));
};

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();





var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};



var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};









var objectWithoutProperties = function (obj, keys) {
  var target = {};

  for (var i in obj) {
    if (keys.indexOf(i) >= 0) continue;
    if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
    target[i] = obj[i];
  }

  return target;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};





var slicedToArray = function () {
  function sliceIterator(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"]) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  return function (arr, i) {
    if (Array.isArray(arr)) {
      return arr;
    } else if (Symbol.iterator in Object(arr)) {
      return sliceIterator(arr, i);
    } else {
      throw new TypeError("Invalid attempt to destructure non-iterable instance");
    }
  };
}();

var RowBoat = function (_React$Component) {
  inherits(RowBoat, _React$Component);

  function RowBoat(props) {
    classCallCheck(this, RowBoat);

    var _this = possibleConstructorReturn(this, (RowBoat.__proto__ || Object.getPrototypeOf(RowBoat)).call(this, props));

    _initialiseProps.call(_this);

    var index = props.defaultIndex;

    _this.state = {
      index: index,
      containerRect: {
        width: 0,
        height: 0
      },
      wrapperStyle: {} // 包装元素style
    };
    _this.mouse = {
      clientXOrigin: 0,
      clientYOrigin: 0,
      diffX: 0, // 拖动后的距离
      diffY: 0 // 拖动后的距离
    };
    _this.wrapper = {
      x: 0,
      y: 0,
      'transition-duration': ''
    };
    return _this;
  }

  createClass(RowBoat, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      var _props = this.props,
          autoplay = _props.autoplay,
          drag = _props.drag;

      this.setRect();
      autoplay && this.setAutoPlay(true);
      window.addEventListener("resize", this._resize);
      if (drag) {
        window.addEventListener('mouseup', this._windowMouseUp);
        this._containerNode.addEventListener('mousedown', this._containerMouseDown);
        window.addEventListener('mousemove', this._windowMouseMove);
      }
    }
  }, {
    key: "componentDidUpdate",
    value: function componentDidUpdate(prevProps) {
      var autoplay = this.props.autoplay;

      if (prevProps.autoplay !== autoplay) {
        autoplay && !this.timer ? this.setAutoPlay(true) : this.setAutoPlay(false);
      }
    }
  }, {
    key: "getStateAndMethod",
    value: function getStateAndMethod() {
      var index = this.state.index;
      var getWrapperProps = this.getWrapperProps,
          getContainerProps = this.getContainerProps,
          getItemProps = this.getItemProps,
          setIndex = this.setIndex;


      return {
        index: index,
        getWrapperProps: getWrapperProps,
        getContainerProps: getContainerProps,
        getItemProps: getItemProps,
        setIndex: setIndex
      };
    }
    // 获取容器元素的属性

    // 获取包装元素的属性

  }, {
    key: "render",
    value: function render() {
      var children = this.props.children;

      var element = children(this.getStateAndMethod());

      if (!element) return null;
      return cloneElement(element);
    }
  }]);
  return RowBoat;
}(React.Component);

RowBoat.propTypes = {
  children: PropTypes.func,
  defaultIndex: PropTypes.number,
  direction: PropTypes.string,
  setDirectionStyle: PropTypes.func,
  length: PropTypes.number,
  speed: PropTypes.number,
  loop: PropTypes.bool,
  autoplay: PropTypes.bool,
  delay: PropTypes.number,
  drag: PropTypes.bool
};
RowBoat.defaultProps = {
  children: noop,
  defaultIndex: 1,
  direction: HORIZONTAL,
  speed: 300,
  delay: 3000
};

var _initialiseProps = function _initialiseProps() {
  var _this2 = this;

  this._containerMouseDown = function (e) {
    var wrapperStyle = _this2.state.wrapperStyle;

    var matrix = parseMatrix(getStyle(_this2._wrapperNode, 'transform')) || [0, 0, 0, 0, 0, 0];

    var _matrix = slicedToArray(matrix, 6),
        x = _matrix[4],
        y = _matrix[5];

    _this2.wrapper.x = +x;
    _this2.wrapper.y = +y;
    _this2.mouseDown = true;
    _this2.mouse.clientXOrigin = e.clientX;
    _this2.mouse.clientYOrigin = e.clientY;

    _this2.setState({
      wrapperStyle: _extends({}, wrapperStyle, {
        transitionDuration: '0ms'
      })
    });
  };

  this._windowMouseMove = function (e) {
    if (!_this2.mouseDown) return;
    var direction = _this2.props.direction;
    var wrapperStyle = _this2.state.wrapperStyle;
    var _wrapper = _this2.wrapper,
        x = _wrapper.x,
        y = _wrapper.y;
    var _mouse = _this2.mouse,
        clientXOrigin = _mouse.clientXOrigin,
        clientYOrigin = _mouse.clientYOrigin;
    var clientX = e.clientX,
        clientY = e.clientY;

    var diffX = reversalNumber(clientXOrigin - clientX);
    var diffY = reversalNumber(clientYOrigin - clientY);

    _this2.mouse.diffX = diffX;
    _this2.mouse.diffY = diffY;

    var translate = direction === HORIZONTAL ? "translate3d(" + (x + diffX) + "px, 0px, 0px)" : "translate3d(0px, " + (y + diffY) + "px, 0px)";
    _this2.setState({
      wrapperStyle: _extends({}, wrapperStyle, {
        transform: translate
      })
    });
  };

  this._windowMouseUp = function (e) {
    if (!_this2.mouseDown) return;
    var direction = _this2.props.direction;
    var _state = _this2.state,
        index = _state.index,
        wrapperStyle = _state.wrapperStyle;
    var _mouse2 = _this2.mouse,
        diffX = _mouse2.diffX,
        diffY = _mouse2.diffY;
    var _state$containerRect = _this2.state.containerRect,
        width = _state$containerRect.width,
        height = _state$containerRect.height;

    var stepX = getStep(diffX, width);
    var stepY = getStep(diffY, height);
    _this2.mouseDown = false;

    if (!diffX && direction === HORIZONTAL) return;
    if (!diffY && direction === VERTICAL) return;
    var step = direction === HORIZONTAL ? stepX : stepY;
    var diff = direction === HORIZONTAL ? diffX : diffY;
    _this2.setState({
      wrapperStyle: _extends({}, wrapperStyle, {
        transitionDuration: _this2.props.speed + "ms"
      })
    }, function () {
      _this2.setIndex(diff >= 0 ? index - step : index + step);
    });
    _this2.mouse.diffX = 0;
    _this2.mouse.diffY = 0;
  };

  this._resize = function () {
    _this2.setRect();
  };

  this.setRect = function () {
    var _containerNode$getBou = _this2._containerNode.getBoundingClientRect(),
        width = _containerNode$getBou.width,
        height = _containerNode$getBou.height;

    _this2.setState({
      containerRect: {
        width: width,
        height: height
      }
    });
  };

  this.setAutoPlay = function (isSet) {
    var delay = _this2.props.delay;

    if (isSet) {
      _this2.timer = setInterval(function () {
        var index = _this2.state.index;
        var _props2 = _this2.props,
            loop = _props2.loop,
            length = _props2.length;

        if (!loop && index === length) {
          clearInterval(_this2.timer);
          _this2.timer = null;
        }
        _this2.setIndex(index + 1);
      }, delay);
    } else {
      clearInterval(_this2.timer);
      _this2.timer = null;
    }
  };

  this.getContainerNode = function (node) {
    _this2._containerNode = node;
  };

  this.getWrapperNode = function (node) {
    _this2._wrapperNode = node;
  };

  this.getContainerProps = function () {
    var _babelHelpers$extends;

    var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    var _ref$refKey = _ref.refKey,
        refKey = _ref$refKey === undefined ? "ref" : _ref$refKey,
        ref = _ref.ref,
        style = _ref.style,
        rest = objectWithoutProperties(_ref, ["refKey", "ref", "style"]);

    return _extends((_babelHelpers$extends = {}, defineProperty(_babelHelpers$extends, refKey, packRefs(ref, _this2.getContainerNode)), defineProperty(_babelHelpers$extends, "style", _extends({
      overflow: "hidden",
      width: "100%",
      height: "100%",
      position: "relative"
    }, style)), _babelHelpers$extends), rest);
  };

  this.getWrapperProps = function () {
    var _babelHelpers$extends2;

    var _ref2 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    var _ref2$refKey = _ref2.refKey,
        refKey = _ref2$refKey === undefined ? "ref" : _ref2$refKey,
        ref = _ref2.ref,
        style = _ref2.style,
        rest = objectWithoutProperties(_ref2, ["refKey", "ref", "style"]);
    var wrapperStyle = _this2.state.wrapperStyle;


    return _extends((_babelHelpers$extends2 = {}, defineProperty(_babelHelpers$extends2, refKey, packRefs(ref, _this2.getWrapperNode)), defineProperty(_babelHelpers$extends2, "style", _extends({}, wrapperStyle, {
      display: "flex",
      width: "100%",
      height: "100%",
      transitionProperty: "transform",
      boxSizing: "content-box"
    }, style)), _babelHelpers$extends2), rest);
  };

  this.getItemProps = function () {
    return {
      style: {
        display: "flex",
        flexShrink: 0,
        width: "100%",
        height: "100%"
      }
    };
  };

  this.setIndex = function (index) {
    var containerRect = _this2.state.containerRect;
    var _props3 = _this2.props,
        length = _props3.length,
        loop = _props3.loop,
        speed = _props3.speed,
        direction = _props3.direction,
        drag = _props3.drag;

    var setStyle = _this2.props.setDirectionStyle || setDirectionStyle;

    if (drag && index > length) index = length;
    if (drag && index < 1) index = 1;
    if (loop && !drag && index > length) index = 1;
    if (loop && !drag && index < 1) index = length;

    var wrapperStyle = setStyle(index, direction, speed, containerRect);
    _this2.setState({
      index: index,
      wrapperStyle: wrapperStyle
    });
  };
};

export { HORIZONTAL, VERTICAL };
export default RowBoat;
